package multinomad.tools;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import multinomad.config.Configuration;
import multinomad.individuals.Population;

public class Function {
	private int findex = -1;
	private List<ClosedInterval.Double > bounds;
	private int dimension;
	
	private Function() {
	
	}
	
	public Function(int index) {
		findex = index;
		
		try {
			
			// Extracting dimensions
			String cmd = "python nomad.py -f "+findex+" -d";
			Process p = Runtime.getRuntime().exec(cmd);
			BufferedReader stdInput = new BufferedReader(new 
	                 InputStreamReader(p.getInputStream()));

            // read the output from the command
			dimension = Integer.parseInt(stdInput.readLine());
			
			// Extracting bounds
			cmd = "python nomad.py -f "+findex+" -b";
			p = Runtime.getRuntime().exec(cmd);
			stdInput = new BufferedReader(new 
	                 InputStreamReader(p.getInputStream()));
			
			bounds = parseBounds(stdInput.readLine());
			
		} catch (IOException e) {e.printStackTrace();}
		
	}
	
	private List<ClosedInterval.Double > parseBounds(String str){
		List<ClosedInterval.Double > listOfBounds = new ArrayList<ClosedInterval.Double >();
		
		
		Pattern lowerUpper = Pattern.compile("\\[|\\]");
		String[] lowerUpperBounds = lowerUpper.split(str);

		Pattern values = Pattern.compile(" ");
		String[] lowerBounds = values.split(lowerUpperBounds[1]);
		String[] upperBounds = values.split(lowerUpperBounds[3]);
		
		for(int i=0;i<lowerBounds.length;i++) {
			double lb = Double.parseDouble(lowerBounds[i]);
			double ub = Double.parseDouble(upperBounds[i]);
			//System.out.println(lb+"-"+ub);
			ClosedInterval.Double aBound = new ClosedInterval.Double(lb, ub);
			listOfBounds.add(aBound);
		}
		return listOfBounds;
	}

	public List<ClosedInterval.Double > getBounds() {
		return bounds;
	}
	
	public int getDimension(){
		return dimension;
	}
	
	public double evaluate(double[] x) {

		//TODO: increment the number of evaluations
		
		//Fitness value to calculate
		double fitness = 0;

		//Parse double[] x as a string for python call
		String doubleAsString=""+x[0];
		for(int i=1;i<x.length;i++)
			doubleAsString+=","+x[i];
		
		//Calling python cec2013 to evaluate the 
		try {
			
			String cmd = "python nomad.py -f "+findex+" -e "+doubleAsString;
			Process p = Runtime.getRuntime().exec(cmd);
			BufferedReader stdInput = new BufferedReader(new 
	                 InputStreamReader(p.getInputStream()));

            // read the output from the command
			fitness = Double.parseDouble(stdInput.readLine());
			
		} catch (IOException e) {e.printStackTrace();}
		return fitness;
	}
	
	public Seeds how_many_goptima(Population p, double accuracy){
		return new Seeds(p, findex, accuracy);
	}

	public static void main(String[] args) {
		Function f = new Function();
		List<ClosedInterval.Double> a = f.parseBounds("[2. 3. 1.3e-2] [25.5 9 2e10]");
		
		for (ClosedInterval.Double doub : a) {
			System.out.println(doub.lower+" to "+doub.upper);
		}
			
	}
}
